/******************************************************************************
* Include files
******************************************************************************/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#define F_CPU 8000000UL
#include <util/delay.h>

#include "DS1820.h"
#include "lcd.h"

/******************************************************************************
* Macros
******************************************************************************/


/******************************************************************************
* Constants
******************************************************************************/


/******************************************************************************
* Global Variables
******************************************************************************/


/******************************************************************************
* External Variables
******************************************************************************/


/******************************************************************************
* Local Function Declarations
******************************************************************************/
void Set_PB0_dir(int dir);
void Set_PB0_level(int level);
uint8_t Read_PB0_input(void);
void DS1820_write_byte(uint8_t byte);
uint8_t DS1820_init(void);
/******************************************************************************
* Local Function Definitions
******************************************************************************/
/******************************************************************************
* Function:         void Set_PB0_dir(int dir)
* Description:		BP0 irny belltsa
* Input:
* Output:
* Notes:
******************************************************************************/
void Set_PB0_dir(int dir)
{
	if(dir)
	{
		DDRB|=(1<<PB0);
	}
	else
	{
		DDRB&=!(1<<PB0);
	}
}
/******************************************************************************
* Function:         void Set_PB0_level(int level)
* Description:		BP0 vezrlse
* Input:
* Output:
* Notes:
******************************************************************************/
void Set_PB0_level(int level)
{
	if(level)
	{
		PORTB|=(1<<PB0);
	}
	else
	{
		PORTB&=!(1<<PB0);
	}
}
/******************************************************************************
* Function:         uint8_t Read_PB0_input(void)
* Description:		BP0 rtknek olvassa
* Input:
* Output:
* Notes:
******************************************************************************/
uint8_t Read_PB0_input(void)
{
	return (PINB & (1<<PB0));
}

/******************************************************************************
* Function:         void DS1820_write_byte(uint8_t byte)
* Description:		byte rsa a 1-wire hlzatra
* Input:
* Output:
* Notes:
******************************************************************************/
void DS1820_write_byte(uint8_t byte)
{
	Set_PB0_level(HIGH);
	Set_PB0_dir(OUTPUT);
	
	for(uint8_t i=0; i<8; i++)
	{
		
		if(byte & (1<<i))
		{
			Set_PB0_level(LOW);
			_delay_us(2);
			Set_PB0_level(HIGH);
			_delay_us(60);
		}
		else
		{
			Set_PB0_level(LOW);
			_delay_us(60);
			Set_PB0_level(HIGH);
			_delay_us(2);	
		}
		
		Set_PB0_level(HIGH);
	}
}

/******************************************************************************
* Function:         uint8_t DS1820_init(void)
* Description:		init szekvencia
* Input:
* Output:
* Notes:
******************************************************************************/
uint8_t DS1820_init(void)
{
	//Set port to output and send master reset (low level for 500 us)
	Set_PB0_dir(OUTPUT);
	Set_PB0_level(LOW);
	_delay_us(550);
	//Set port to input and waiting for 80 us
	Set_PB0_dir(INPUT);
	_delay_us(80);
	//Read port, DS1820 presence is okay, if port is pulled low
	uint8_t status = !Read_PB0_input();
	_delay_us(100);
	return status;
	
}

/******************************************************************************
* Global Function Definitions
******************************************************************************/


/******************************************************************************
* Function:         int16_t DS1820_read_temperature(void)
* Description:		hmrsklet kiolvassa
* Input:
* Output:
* Notes:
******************************************************************************/
int16_t DS1820_read_temperature(void)
{
	uint8_t byte_array[9];
	uint8_t byte_counter=0;
	for(uint8_t i=0; i<9; i++) byte_array[i]=0;

	_delay_ms(50);
	DS1820_init();
	//Skip ROM (if only one device exists on the 1-Wire bus)
	DS1820_write_byte(0xCC);
	//Convert T (Starting temperature measuring
	DS1820_write_byte(0x44);
	_delay_ms(100);
	DS1820_init();
	//Skip ROM (if only one device exists on the 1-Wire bus)
	DS1820_write_byte(0xCC);
	//Read Scratchpad (9 bytes)
	DS1820_write_byte(0xBE);
	_delay_us(500);
	for(uint8_t i=0; i<72; i++)
	{
		Set_PB0_dir(OUTPUT);
		Set_PB0_level(LOW);
		_delay_us(2);
		Set_PB0_dir(INPUT);
		_delay_us(15);
		byte_array[byte_counter] |= Read_PB0_input()<<(i-(byte_counter*8));
		_delay_us(30);
		if((i+1) % 8 == 0) byte_counter++;
	}
	return (byte_array[0] | (byte_array[1]<<8));
}



